package com.isu.gaswellwatch.modbus.data.impl;

import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.stereotype.Component;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Map;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/24 17:33
 */
@Component(PersistenceHandler.SCSS_MODBUS_TYPE)
public class ScssPersistenceHandler extends AbstractPersistenceHandler {

    @Override
    public void createTable(String tableName, Long deviceId) {
        this.createTable("sql/CREATE_SCSS.sql", tableName, deviceId);
    }

    @Override
    public Map<String, Map<String, Object>> insert(String tableName, String cacheKey) {
        String insertTableSQL = this.getResource("sql/INSERT_SCSS.sql");
        insertTableSQL = StringUtils.replace(insertTableSQL, "$TableName$", tableName);

        Map<String, Object> oldRow = this.getLastRow(tableName);
        Map<String, Object> newRow = this.redisTemplate.opsForHash().entries(cacheKey);

        this.jdbcTemplate.execute(insertTableSQL, new PreparedStatementCallback<Object>() {
            @Override
            public Object doInPreparedStatement(PreparedStatement ps) throws SQLException {
                ps.setLong(1, ScssPersistenceHandler.this.snowflakeConfig.snowflakeId());
                ScssPersistenceHandler.this.setValue(ps, newRow, 2, "deviceId", Types.BIGINT);
                ScssPersistenceHandler.this.setValue(ps, newRow, 3, "collectionTime", Types.TIMESTAMP);
                ScssPersistenceHandler.this.setValue(ps, newRow, 4, "receiveTime", Types.TIMESTAMP);
                ScssPersistenceHandler.this.setValue(ps, newRow, 5, "casPressure", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 6, "oilPressure", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 7, "firstSolenoidStatus", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 8, "batteryVoltage", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 9, "solarVoltage", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 10, "remainingTimeAction", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 11, "secondSolenoidStatus", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 12, "preTransmission", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 13, "internetTraffic", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 14, "loadFactor", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 15, "dataTime", Types.TIMESTAMP);
                ScssPersistenceHandler.this.setValue(ps, newRow, 16, "showDelay", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 17, "openWellSamplingInterval", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 18, "closeWellSamplingInterval", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 19, "ctlModel", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 20, "minPressure", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 21, "maxPressure", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 22, "pressureMinVoltage", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 23, "pressureMaxVoltage", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 24, "oilMin", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 25, "oliMax", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 26, "oilMinVoltage", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 27, "oilMaxVoltage", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 28, "inputPressureMinValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 29, "inputPressureMaxValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 30, "inputVoltageMinValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 31, "inputVoltageMaxValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 32, "flowRateMinValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 33, "flowRateMaxValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 34, "flowVoltageMinValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 35, "flowVoltageMaxValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 36, "continuousSamplingIntervalDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 37, "sensorSignalEffectiveLevel", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 38, "pressureCompensationPolarityFlag", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 39, "pressureCompensationValueSetting", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 40, "oilPressureCompensationPolarityFlag", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 41, "oilPressureCompensationValueSetting", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 42, "inputPressureCompensationPolarityFlag", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 43, "inputPressureCompensationValueSetting", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 44, "flowCompensationPolarityFlag", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 45, "flowCompensationValueSetting", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 46, "wellOpenTimeTimestamp", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 47, "wellOpenPressureValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 48, "wellOpenOilPressureValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 49, "wellOpenLoadFactorPresets", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 50, "wellCloseTimeTimestamp", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 51, "wellClosePressureValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 52, "wellCloseOilPressureValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 53, "wellCloseFlowValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 54, "minWellOpenTimeDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 55, "maxWellOpenTimeDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 56, "minWellCloseTimeDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 57, "maxWellCloseTimeDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 58, "pressureStabilizationDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 59, "flowStabilizationDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 60, "loadFactorStabilizationDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 61, "plungerDelayDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 62, "plungerRiseDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 63, "continuosFlowDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 64, "wellCloseTimeDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 65, "wellCloseTimeNotReachedDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 66, "wellCloseNotReachedCountValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 67, "plungerDelayDurationRepeat", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 68, "targetTimeTimestamp", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 69, "targetTimeRangeValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 70, "continuosFlowIncreaseDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 71, "continuosFlowDecreaseDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 72, "wellCloseIncreaseDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 73, "wellCloseDecreaseDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 74, "minWellCloseTimeDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 75, "maxWellCloseTimeDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 76, "minContinuosFlowTimeDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 77, "maxContinuosFlowTimeDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 78, "minWellOpenTimeDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 79, "maxWellOpenTimeDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 80, "wellOpenPressureValueAtOpen", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 81, "wellOpenOilPressureValueAtOpen", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 82, "wellOpenLoadFactorPresetsAtOpen", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 83, "wellClosePressureValueAtClose", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 84, "wellCloseOilPressureValueAtClose", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 85, "wellCloseFlowValueAtClose", Types.INTEGER);
                return ps.executeUpdate();
            }
        });
        if (Objects.isNull(oldRow)) {
            return Map.of("new", newRow);
        }
        return Map.of("new", newRow, "old", oldRow);
    }
}
